#!/usr/bin/env python

'''

    @version $Id: map.py 7263 2007-05-25 18:41:58Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import math

from xml import dom

from libacme import resource
from libacme.mapobjects import wall
from libacme.mapobjects import spawn
from libacme.mapobjects import zone

# this class contains the map

class aaMap(resource.Resource):
    __mapObjects = None
    
    def __init__(self, theResource=None):
        resource.Resource.__init__(self, theResource.GetDom() )
        self.__mapObjects = []
        
        if theResource is not None:
            self.CopyResource(theResource)
            self.Parse(theResource.GetDom() )
            
    def GetObjects(self):
        if self.__mapObjects is not None:
            return self.__mapObjects
        
        return []
            
    def UpdateFromDom(self):
        self.__mapObjects = []
        self.Parse()
            
    def Parse(self, theResource=None):
        if theResource is None:
            theResource = self.theDom
        theObjects = theResource.getElementsByTagName("Field")[0].childNodes
        
        for a in theObjects:
            if a.nodeType == dom.Node.ELEMENT_NODE:
                if a.tagName == "Spawn":
                    self.ParseSpawn(a)
                elif a.tagName == "Wall":
                    self.ParseWall(a)
                elif a.tagName == "Zone":
                    self.ParseZone(a)
                
    def ParseSpawn(self, theSpawn):
        spawnDict = {
                                'x' : theSpawn.getAttribute('x'),
                                'y' : theSpawn.getAttribute('y'),
                                'xdir' : theSpawn.getAttribute('xdir'),
                                'ydir' : theSpawn.getAttribute('ydir'),
                                'angle' : theSpawn.getAttribute('angle')
                                }
        
        # special case for an empty spawn tag, also make them all the right types
        if spawnDict['x'] == "":
            spawnDict['x'] = 0.0
        else:
            spawnDict['x'] = float(spawnDict['x'])
        if spawnDict['y'] == "":
            spawnDict['y'] = 0.0
        else:
            spawnDict['y'] = float(spawnDict['y'])
        
        hasxdir = False
        if spawnDict['xdir'] == "":
            pass
        else:
            hasxdir = True
            spawnDict['xdir'] = float(spawnDict['xdir'])
        
        hasydir = False
        if spawnDict['ydir'] == "":
            pass
        else:
            hasxdir = True
            spawnDict['ydir'] = float(spawnDict['ydir'])
            
        hasangle = False
        if spawnDict['angle'] != "":
            hasangle = True
            spawnDict['angle'] = float(spawnDict['angle'])

        self.__mapObjects.append( spawn.xSpawn(spawnDict, theSpawn) )

    def ParseWall(self, theWall):
        points = []
        
        for a in theWall.getElementsByTagName("Point"):
            points.append( [ float(a.getAttribute("x")), float(a.getAttribute("y")) ] )
            
        self.__mapObjects.append( wall.xWall(points) )
        
    def ParseZone(self, theZone):
        zoneDict = {
                    'effect' : theZone.getAttribute("effect")
        }

        theCircle = theZone.getElementsByTagName("ShapeCircle")[0]
        zoneDict['radius'] = float(theCircle.getAttribute("radius") )
        zoneDict['growth'] = float(theCircle.getAttribute("growth") )
        
        thePos = theCircle.getElementsByTagName("Point")[0]
        
        zoneDict['x'] = float(thePos.getAttribute("x") )
        zoneDict['y'] = float(thePos.getAttribute("y") )
        
        self.__mapObjects.append( zone.xZone(zoneDict) )

