#!/usr/bin/env python

'''

    @version $Id: base_element.py 7238 2007-05-21 07:45:07Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''


# this is the base class for all map elements
class baseElement(object):
    '''
        This is the base class for all map elements.  You are expected to
        derive from this class for new map elements.

        @ivar tagname: The name of the xml tag that represents this element
        @type tagname: String
        @ivar attrList: A dictionary where the keys are possible attributes
           and each item is a list of possible values or a string containing
           a type.
        @type attrList: Dictionary
        @ivar subtags: A list containing xml tags that this element can contain
        @type subtags: List
        @ivar children: The child tags that this element contains.
        @type children: List
        
    '''
    tagname = None
    attrList = None
    subtags = None
    children = None
    
    x = None
    y = None
    
    def __init__(self, **args):
        '''
           The constructor for base elements.
        '''
        super(baseElement, self).__init__(**args)
        
        self.attrList = {}
        
        self.x = 0
        self.y = 0

    def AddAttribute(self, attribute, values):
        '''
           Add an attribute to this element.

           You should call this method from your derived class's init method
           to register all attributes it supports.

           @param attribute: The attribute you're adding.
           @type attribute: String
           @param values:  A list of possible values for this attribute.  It may
              also be a string representing a single type, for example "numeric"
           @type values: List or String
        '''
        attribute = attribute.lower()
        
        self.attrList[attribute] = values

           