#!/usr/bin/env python

'''

    @version $Id: __init__.py 7231 2007-05-20 22:53:21Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import base_element

class xWall(base_element.baseElement):
    objType = "wall"
    
    def __init__(self, points, **args):
        super(xWall, self).__init__(**args)
                
        firstTime = True
        
        leftx = None
        rightx = None
        topy = None
        bottomy = None
        
        for a in points:
            # Since we can't predict where people will put walls, we just read the first point and find the
            # bounding rect from there
            if firstTime is True:
                leftx, bottomy = a
                rightx, topy = a
                firstTime = False
            else:
                if a[0] < leftx:
                    leftx = a[0]
                if a[0] > rightx:
                    rightx = a[0]
                
                if a[1] < bottomy:
                    bottomy = a[1]
                if a[1] > topy:
                    topy = a[1]

        # store the position, this is only used once, after object creation
        self.x = (rightx - leftx)/2
        self.y = (topy-bottomy)/2
        
        # Compute the bounding rect
        self.leftx = leftx-self.x
        self.rightx = rightx-self.x
        self.topy = topy-self.y
        self.bottomy = bottomy-self.y
        
        self.points = []
        
        # Now adjust the points for a 0,0 origin internally.
        for a in points:
            self.points.append( [ a[0] - self.x, a[1] - self.y ] )

