#!/usr/bin/env python

'''

    @version $Id: paths.py 7236 2007-05-21 05:01:54Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005 by David Fancella

'''

import sys, os

__allPaths = {}

def InitializePaths():
    global __allPaths

    if sys.platform == 'win32':
        __home = os.path.expanduser('~/')
        __basepath = ''
        __share = ''
        __lib = ''
        __etc = 'etc'
    else:
        __basepath = sys.path[0]
        __home = os.path.expanduser('~')
        # installed to some system prefix
        if __basepath.startswith(sys.prefix):
            __share = os.path.join('share', 'acme')
            __lib = os.path.join('lib', 'libacme')
            __etc = 'etc'
        # installed to any arbitrary prefix
        if  __basepath.endswith('/lib'):
            __basepath = os.path.dirname(__basepath)
            __share = os.path.join('share', 'acme')
            __lib = os.path.join('lib', 'libacme')
            __etc = 'etc'
        # run from source directory
        else:
            __share = ''
            __lib = ''
            __etc = 'etc'
            
    __allPaths = { 'resource' : os.path.join(__basepath, __share, "resources"),
                   'graphics' : os.path.join(__basepath, __share, "resources", "graphics"),
                   'icons' : os.path.join(__basepath, __share, 'resources', 'icons'),
                   'fonts' : os.path.join(__basepath, __share, "resources", "fonts"),
                   'dtd' : os.path.join(__basepath, __share, "resources", "dtd"),
                   'ui' : os.path.join(__basepath, __share, "resources", "ui"),
                   'default' : os.path.join(__home, "armagetronmaps") }

# todo: make GetPath throw an exception or something instead if a path isn't known
def GetPath(path, targetFile=None):
    global __allPaths
    
    if __allPaths.has_key(path):
        if targetFile is None:
            return __allPaths[path]
        else:
            return JoinPaths(__allPaths[path], targetFile)
    else:
        print "Don't know path '" + path + "'!"
        print __allPaths
        return None

def JoinPaths(apath, bpath):
    return os.path.join(apath, bpath)

def GetIcon(theIcon):
    return JoinPaths( GetPath('icons'), theIcon)

def SearchModulePath(resource, file, module=None):
    # first search in the module's specific graphics directory
    if module is not None:
        tmpPath = Join(resource, module)
        if os.path.exists(tmpPath) and os.path.isdir(tmpPath):
            newPath = os.path.join(tmpPath, file)
            if os.path.exists(newPath) and os.path.isfile(newPath):
                return newPath
    
    newPath = Join(resource, file)
    if os.path.exists(newPath) and os.path.isfile(newPath):
        return newPath

    raise IOError, file + " not found"
    

        
        