#!/usr/bin/env python

'''

    @version $Id: __init__.py 7231 2007-05-20 22:53:21Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005, 2007 by David Fancella

'''

from PyQt4 import QtGui

lastid = 0

class EditorBase(object):
    toolBoxParent = None
    __id = None
    
    def __init__(self, toolBoxParent):
        self.toolBoxParent = toolBoxParent
        global lastid
        lastid += 1
        self.__id = lastid
        
    def id(self):
        return self.__id
        
    def idstr(self):
        return str(self.__id)

    # Override this to tell the parent your document has changes
    def HasChanges(self):
        print "EditorBase: HasChanges not implemented yet"
        return True

    # CommitToDom needs to yield values that start at 0 and progress to 100.  It can skip as needed, and
    # these integers should indicate the percentage of dom-committing that's complete.
    def CommitToDom(self):
        print "EditorBase: CommitToDom not implemented yet"
        return [100]
        
    # UpdateFromDom is the inverse of committodom and is required for view source to work properly
    # and synchronize changes between the two widgets.  The expected yield is to be symmetric to CommitToDom
    def UpdateFromDom(self):
        print "EditorBase: UpdateFromDom no implemented yet"
        return [100]
    
    # Override this to provide access to the resource object, which must inherit Resource
    def GetResource(self):
        print "EditorBase: GetResource not implemented yet"
        return None
        
    # Override this to parse the resource.  You will receive an instance of Resource
    def LoadResource(self, thePath):
        print "EditorBase: LoadResource not implemented yet"
        
    # Override this to create a new resource for the editor
    def NewResource(self):
        print "EditorBase: NewResource not implemented yet"

    # Return a dict that contains the actions
    def GetActions(self):
        print "EditorBase: GetActions not implemented yet"
        return None
        
    # Return a QWidget subclass that will be placed in the toolbox area on the right of the ui
    def GetToolBox(self):
        print "EditorBase: GetToolBox not implemented yet"
        return QtGui.QWidget(self.toolBoxParent)


