#!/usr/bin/env python

'''

    @version $Id: __init__.py 7231 2007-05-20 22:53:21Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005, 2007 by David Fancella

'''

from PyQt4 import QtGui, QtCore

from libacme import map
from libacme import paths

from libacmegui.editors import editorbase
from libacmegui.editors.mapwidgets import wall
from libacmegui.editors.mapwidgets import spawn
from libacmegui.editors.mapwidgets import zone

class CrossHairs(QtGui.QGraphicsItem):
    def __init__(self):
        QtGui.QGraphicsItem.__init__(self)
        self.setPos( 0,0 )
        self.setZValue(100)
        self.setAcceptedMouseButtons(QtCore.Qt.NoButton)
        
    def boundingRect(self):
        return QtCore.QRectF( -36, -36, 72, 72 )

    def paint(self, painter, option, widget):
        frontBrush = QtGui.QBrush(QtGui.QColor( 0, 255, 0) )
        painter.setBrush(frontBrush)
        frontPen = QtGui.QPen(QtCore.Qt.SolidLine)
        frontPen.setBrush(frontBrush)
        frontPen.setWidth(1)
        painter.setPen(frontPen)
        painter.drawLine( -26, 0, 26, 0)
        painter.drawLine( 0, -26, 0, 26)
        
class MapScene(QtGui.QGraphicsScene):
    def __init__(self, parent):
        QtGui.QGraphicsScene.__init__(self, parent)
        #self.snapPointer = CrossHairs()
        
        #self.addItem(self.snapPointer)

    def mouseMoveEvent(self, eve):
        #self.snapPointer.setPos( eve.scenePos() )
        #self.snapPointer.update()
        QtGui.QGraphicsScene.mouseMoveEvent(self, eve)

class mapView(QtGui.QGraphicsView, editorbase.EditorBase):
    def __init__(self, parent, toolboxparent):
        QtGui.QGraphicsView.__init__(self, parent)
        editorbase.EditorBase.__init__(self, toolboxparent)
        
        self.setCursor(QtCore.Qt.CrossCursor)
        #self.setViewportMargins(3,3,3,3)
        self.setInteractive(True)
        
        self.currentMousePos = (0, 0)
        
        self.mapScene = MapScene(self)
        self.setScene(self.mapScene)
        self._mode = "view"
        
        self.show()
        self.theMap = None
        self.transformation = QtGui.QMatrix()
        self.transformation.scale(1,-1)
        self.setMatrix(self.transformation)
        self.scaleFactor = 1
        QtCore.QObject.connect(self.mapScene, QtCore.SIGNAL("changed(const QList<QRectF>&)"), self.updateSceneP)
        
        self._createActions()
        
    def _createActions(self):
        self.actionList = { 'name' : 'Map',
                                    'actions' : QtGui.QActionGroup(self) }

        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetPath('icons', 'walls.png') ) ), "Add Wall", self.actionList['actions'])
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.notImpl)
        self.actionList['actions'].addAction(tmpAction)
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('spawn.png') ) ), "Add Spawn", self.actionList['actions'])
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.notImpl)
        self.actionList['actions'].addAction(tmpAction)
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('editcut.png') ) ), "Slash", self.actionList['actions'])
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.notImpl)
        self.actionList['actions'].addAction(tmpAction)
                
    def GetActions(self):
        return self.actionList
        
    def GetResource(self):
        return self.theMap
        
    def notImpl(self):
        print "This feature not yet implemented, whatever it is"
        
    def updateSceneP(self, theRect):
        self.updateScene(theRect)
        
    def LoadResource(self, theMap):
        self.theMap = map.aaMap(theMap)
                
        self.UpdateFromDom()
        
        self.scale(1,1)

    def UpdateFromDom(self):
        for a in self.mapScene.items():
            self.mapScene.removeItem(a)
        
        self.theMap.UpdateFromDom()
            
        for anObj in self.theMap.GetObjects():
            if anObj.objType == "wall":
                self.mapScene.addItem( wall.Wall(anObj) )
            elif anObj.objType == "spawn":
                self.mapScene.addItem( spawn.Spawn( anObj) )
            elif anObj.objType == "zone":
                self.mapScene.addItem( zone.Zone(anObj) )
        
        self.mapScene.update()
        
    def mouseMoveEvent(self, e):
        if self._mode == "translate":
            #print self.mouseStart[0] - e.x(), self.mouseStart[1] - e.y()
            #print self.horizontalScrollBar().maximum()
            #print self.verticalScrollBar().maximum()
            #self.mapScene.translate( self.mouseStart[0] - e.x(), self.mouseStart[1] - e.y() )
            pass
        self.emit(QtCore.SIGNAL("mouseHover"), self.mapToScene( e.pos() ) )
        self.currentMousePos = ( e.x(), e.y() )
        QtGui.QGraphicsView.mouseMoveEvent(self, e)

    def mousePressEvent(self, e):
        if e.button() == QtCore.Qt.MidButton:
            self._mode = "translate"
            self.mouseStart = [e.x(), e.y()]
        QtGui.QGraphicsView.mousePressEvent(self, e)

    def mouseReleaseEvent(self, e):
        if self._mode == "translate":
            if e.button() == QtCore.Qt.MidButton:
                self.centerOn( self.mapToScene( e.pos() ) )
        self._mode = "view"
        QtGui.QGraphicsView.mouseReleaseEvent(self, e)
        
    def wheelEvent(self, e):
        scaleFactorDelta = float(e.delta() )/150.0
        
        if float(self.scaleFactor) + scaleFactorDelta > 0:
            self.scaleFactor = float(self.scaleFactor) - scaleFactorDelta
        #self.scale(1.0/self.scaleFactor, 1.0/self.scaleFactor)
        if scaleFactorDelta > 0:
            self.scale(1.0/scaleFactorDelta, 1.0/scaleFactorDelta)
        else:
            self.scale(-scaleFactorDelta, -scaleFactorDelta)
        QtGui.QGraphicsView.wheelEvent(self, e)
                
    def drawBackground(self, painter, rect):
        backBrush = QtGui.QBrush(QtGui.QColor( 0, 0, 0) )
        painter.setBackground(backBrush)
        
        painter.fillRect(rect, backBrush)
        
        frontBrush = QtGui.QBrush(QtGui.QColor( 50, 50, 50) )
        painter.setBrush(frontBrush)
        frontPen = QtGui.QPen(QtCore.Qt.SolidLine)
        frontPen.setBrush(frontBrush)
        frontPen.setWidth(1)
        painter.setPen(frontPen)
        
        leftx, bottomy = rect.x(), rect.y()
        width, height = rect.width(), rect.height()
        
        leftStart = int(leftx)
        while(leftStart % 50 != 0):
            leftStart += 1
            
        bottomStart = int(bottomy)
        while(bottomStart % 50 != 0):
            bottomStart += 1
        
        # Draw grid
        for a in range(int(leftStart), int(leftx + width), 50 ):
            painter.drawLine(a, bottomy, a, bottomy + height)
        for a in range(int(bottomStart), int(bottomy + height), 50 ):
            painter.drawLine(leftx, a, leftx + width, a)
        
        frontPen.setWidthF(3.0)
        painter.setPen(frontPen)
        # Draw axes
        if leftx <= 0 and (leftx + width) >= 0:
            painter.drawLine(0,bottomy, 0, bottomy + height)
        if bottomy <= 0 and (bottomy + height) >= 0:
            painter.drawLine(leftx, 0, leftx + width, 0)
        
        
        
