#!/usr/bin/env python

'''

    @version $Id: __init__.py 7231 2007-05-20 22:53:21Z davidfancella $

    This file is part of acme.

    acme is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    acme is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with acme; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    acme is copyright 2005, 2007 by David Fancella

'''

from PyQt4 import QtGui, QtCore

class Wall(QtGui.QGraphicsItem):
    def __init__(self, wallObj):
        #print type(parent)
        QtGui.QGraphicsItem.__init__(self)
        
        self.setFlags( QtGui.QGraphicsItem.ItemIsMovable or QtGui.QGraphicsItem.ItemIsSelectable)
        self.setAcceptsHoverEvents(True)
        
        # Walls are always drawn first
        self.setZValue(0)
        
        self.setPos(wallObj.x, wallObj.y )
        self.theWall = wallObj
        
        self.hovering = False
        
    def paint(self, painter, option, widget):
        painter.setPen(QtCore.Qt.black);
        thePointList = QtGui.QPolygonF()
        forePen = QtGui.QPen(QtCore.Qt.SolidLine)
        forePen.setColor( QtGui.QColor( 255, 255, 255) )
        forePen.setWidthF(1.8)
        painter.setPen(forePen)
        
        for a in self.theWall.points:
            thePointList.append( QtCore.QPointF(a[0], a[1]) )
        painter.drawPolyline(thePointList )
        
        if self.hovering:
            forePen = QtGui.QPen(QtCore.Qt.SolidLine)
            forePen.setColor( QtGui.QColor( 255, 0, 0) )
            forePen.setWidthF(1)
            theBrush = QtGui.QBrush(QtCore.Qt.RadialGradientPattern )
            theBrush.setColor( QtGui.QColor( 255, 0, 0 ) )
            painter.setBrush( theBrush )
            painter.setPen(forePen)
            for a in self.theWall.points:
                theRect = QtCore.QRectF( a[0]-2, a[1]-2, 4, 4 )
                
                painter.drawEllipse(theRect)
        
    def hoverEnterEvent(self, eve):
        self.hovering = True
        print "Hovering!"
        self.update()
        
    def hoverLeaveEvent(self, eve):
        self.hovering = False
        print "Not Hovering"
        self.update()
        
    def boundingRect(self):
        #print self.leftx, self.bottomy, self.rightx-self.leftx, self.topy-self.bottomy
        return QtCore.QRectF( self.theWall.leftx-5, self.theWall.bottomy-5, self.theWall.rightx-self.theWall.leftx+10, self.theWall.topy-self.theWall.bottomy+10 )

    def shape(self):
        thePointList = QtGui.QPolygonF()
        for a in self.theWall.points:
            thePointList.append( QtCore.QPointF(a[0], a[1]) )
        
        theShape = QtGui.QPainterPath()
        theShape.addPolygon(thePointList)
        return theShape
