#!/usr/bin/env python

'''

    @version $Id: __init__.py,v 1.0 0005/01/14 00:50:36 davidfancella Exp $

    This file is part of Dave's Stupid Alarm Clock.

    Dave's Stupid Alarm Clock is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Dave's Stupid Alarm Clock is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Dave's Stupid Alarm Clock; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  00111-1307  USA

	Dave's Stupid Alarm Clock is copyright 2006 by Dave Fancella

'''

from PyQt4 import QtCore, QtGui
from PyQt4 import uic

from libacme import paths
from libacme import resource_parser
from libacmegui import mainView
from libacmegui.dialogs import GetSaveFile

# Set up widgets
Ui_acmeMain, throwmeaway = uic.loadUiType( paths.JoinPaths( paths.GetPath('ui'), 'main.ui') )

class acmeApp(QtGui.QApplication):
    def __init__(self, args):
        QtGui.QApplication.__init__(self,args)
        
        self.mainwin = MainFrame(None)        
        self.mainwin.show()
        
        self.exec_()

class MainFrame(QtGui.QMainWindow, Ui_acmeMain):
    def __init__(self, args):
        QtGui.QMainWindow.__init__(self, args)

        self.setupUi(self)
        
        self.toolView = mainView.mainView(self)
        
        #self.mapViewer = mapEditor.mapView(self)
        #self.toolView.setCurrentWidget(self.toolView)
        
        self.setCentralWidget(self.toolView)
        
        self._connectSlots()
        self._createFileActions()
        self._createViewActions()
        self._createMenus()
        self._createToolbars()
        self._createStatusBar()
        
        self.setWindowIcon( QtGui.QIcon( paths.GetPath('icons', "armagetronad.png") ) )

    def _connectSlots(self):
        QtCore.QObject.connect( self.toolView, QtCore.SIGNAL("mouseHover"), self.mouseHover)
        QtCore.QObject.connect( self.toolView, QtCore.SIGNAL("editorSelected"), self.EditorSelected)
        QtCore.QObject.connect( self.toolView, QtCore.SIGNAL("activeContentChanged"), self.ActiveContentChanged)
        QtCore.QObject.connect( self.toolView, QtCore.SIGNAL("modificationChanged"), self.ContentChanged)

    def _createFileActions(self):
        self.fileActions = QtGui.QActionGroup(self)
        self.fileActionHash = {}
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('new_map.png') ) ), "&New", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.fileNew)
        self.fileActions.addAction(tmpAction)
        self.fileActionHash['new'] = tmpAction
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('fileopen.png') ) ), "&Open", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.fileOpen)
        self.fileActions.addAction(tmpAction)
        self.fileActionHash['open'] = tmpAction
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('filesave.png') ) ), "&Save", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.fileSave)
        tmpAction.setEnabled(False)
        self.fileActions.addAction(tmpAction)
        self.fileActionHash['save'] = tmpAction
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('filesaveas.png') ) ), "Save &As", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.fileSaveAs)
        self.fileActions.addAction(tmpAction)
        tmpAction.setEnabled(False)
        self.fileActionHash['save_as'] = tmpAction
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('save_all.png') ) ), "Save All", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.fileSaveAll)
        self.fileActions.addAction(tmpAction)
        tmpAction.setEnabled(False)
        self.fileActionHash['save_all'] = tmpAction

        tmpAction = QtGui.QAction(self.fileActions)
        tmpAction.setSeparator(True)
        self.fileActions.addAction(tmpAction)

        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('package.png') ) ), "&Package", self.fileActions)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.filePackage)
        self.fileActions.addAction(tmpAction)
        self.fileActionHash['package'] = tmpAction
        
    def _createViewActions(self):
        self.viewActions = QtGui.QActionGroup(self)
        self.viewActions.setExclusive(False)
        self.viewActionHash = {}
        
        tmpAction = QtGui.QAction( QtGui.QIcon(QtGui.QPixmap( paths.GetIcon('view_source.png') ) ), "&Source", self.viewActions)
        tmpAction.setCheckable(True)
        QtCore.QObject.connect(tmpAction, QtCore.SIGNAL("triggered(bool)"), self.viewSource)
        self.viewActions.addAction(tmpAction)
        self.viewActionHash['source'] = tmpAction

    # for stuff Qt Designer doesn't handle
    def _createToolbars(self):
        self.theFileToolbar = QtGui.QToolBar()
        self.theFileToolbar.setIconSize( QtCore.QSize(32,32) )
        self.theFileToolbar.addActions(self.fileActions.actions() )
        self.addToolBar(self.theFileToolbar)

        self.theViewToolbar = QtGui.QToolBar()
        self.theViewToolbar.setIconSize( QtCore.QSize(32,32) )
        self.theViewToolbar.addActions(self.viewActions.actions() )
        self.addToolBar(self.theViewToolbar)

        self.editorToolbar = QtGui.QToolBar()
        self.editorToolbar.setIconSize( QtCore.QSize(32,32) )
        self.addToolBar(self.editorToolbar)
        self.editorToolbar.hide()
        
    def _createMenus(self):
        self.theFileMenu = QtGui.QMenu("&File")
        self.theFileMenu.addActions(self.fileActions.actions() )
        self.theFileMenu.addSeparator()
        self.theFileMenu.addAction("Preferences", self.filePreferences)
        self.theFileMenu.addSeparator()
        self.theFileMenu.addAction("&Quit", self.quit)
        self.menubar.addMenu(self.theFileMenu)
        
        self.theViewMenu = QtGui.QMenu("&View")
        self.theViewMenu.addActions(self.viewActions.actions() )
        self.menubar.addMenu(self.theViewMenu)

        #self.theEditMenu = QtGui.QMenu("&Edit")
        #self.theEditMenu.addActions(self.editActions.actions() )
        #self.menubar.addMenu(self.theEditMenu)

    def _createStatusBar(self):
        self.statusText = QtGui.QLabel(self)
        self.statusBar().addWidget(self.statusText)

    def quit(self):
        QtGui.QApplication.quit()

    def EditorSelected(self):
        self.ActiveContentChanged()
        
    def ActiveContentChanged(self):
        if self.toolView.ActiveWidgetHasChanges():
            self.fileActionHash['save'].setEnabled(True)
        else:
            self.fileActionHash['save'].setEnabled(False)
        
        if self.toolView.count() > 0:
            self.fileActionHash['save_as'].setEnabled(True)
        else:
            self.fileActionHash['save_as'].setEnabled(False)
            
        if self.toolView.HasAnyChanges():
            self.fileActionHash['save_all'].setEnabled(True)
        else:
            self.fileActionHash['save_all'].setEnabled(False)
            
    def ContentChanged(self):
        self.ActiveContentChanged()

    def SetEditorActions(self, theActions):
        if theActions is not None:
            self.editorToolbar.clear()
            self.editorToolbar.addActions(theActions['actions'].actions() )
            self.editorToolbar.show()
        else:
            self.editorToolbar.hide()

    def RemoveEditorActions(self):
        self.editorToolbar.clear()
        self.editorToolbar.hide()

    def fileNew(self):
        pass
        
    def fileOpen(self):
        fileName = str(QtGui.QFileDialog.getOpenFileName(self, "Open File",
                                                "./",
                                                "Armagetron Advanced Resources (*.xml)") )
        
        if fileName != "":
            self.toolView.LoadResource(str(fileName) )
        
    def fileSave(self):
        self.toolView.SaveActiveResource()
        
    def fileSaveAs(self):
        fileName = GetSaveFile(self)
        
        if fileName != "":
            self.toolView.SaveActiveResource(str(fileName) )

    def fileSaveAll(self):
        self.toolView.SaveAll()

    def filePackage(self):
        pass

    def filePreferences(self):
        pass

    def viewSource(self, toggle):
        self.toolView.SetViewSource(toggle)

    def mouseHover(self, pos):
        self.statusText.setText( str(pos.x()) + ", " + str(pos.y() ) )
